/**********************************************************************/
/* Project Name:  flash_g128.c                                        */
/* Last modified: 09/02/2012                                          */
/* By:            b01802                                              */
/**********************************************************************/
/* Description: Flash routines for microcontroller S12G128            */
/* This software is classified as Engineering Sample Software.        */
/**********************************************************************/
#include "flash_g128.h"

unsigned long int Local2Global(unsigned long int address);
unsigned char PFLASH_Erase_Verify_Section(unsigned long int address, unsigned int number_of_phrases);
unsigned char PFLASH_Program(unsigned long int address, unsigned int *ptr);
unsigned char PFLASH_Erase_Sector(unsigned long int address);
unsigned char PFLASH_Read_Byte(unsigned long int address);
unsigned  int PFLASH_Read_Word(unsigned long int address);
void PFLASH_Init(unsigned char fdiv);

//==============================================================================
//PFLASH Send_Command
//==============================================================================
//this function is stored in RAM memory
//in C language:
//  {
//    FSTAT_CCIF = 1;         //launch command
//    while(FSTAT_CCIF == 0); //wait for done
//  }
static unsigned char Send_Command[]=
{
  0x1C, 0x01, 0x06, 0x80, 0x1F, 0x01, 0x06, 0x80, 0xFB, 0x3D
};

//==============================================================================
//Local2Global
//==============================================================================
unsigned long int Local2Global(unsigned long int address)
{
  //EEPROM
  if((address >= 0x0400) && (address <= 0x13FF))
  {
    address = (address + 0x4000) | 0x00010000;
    return address; 
  }
  
  //Unpaged P-Flash 0x1400
  if((address >= 0x1400) && (address <= 0x27FF))
  {
    address = address | 0x00030000;
    return address; 
  }
  
  //Unpaged P-Flash 0x4000 and 0xC000
  if(((address >= 0x4000) && (address <= 0x7FFF)) || ((address >= 0xC000) && (address <= 0xFFFF)))
  {
    address = address | 0x00030000;
    return address; 
  }

  //Paged P-Flash 0x8000
  if((address >= 0x00028000) && (address <= 0x000FFFFF))
  {
    address = ((address >> 2) & 0x000F0000) | (((((address & 0x000F0000) >> 16)%4) * 0x4000) | (address & 0x00007FFF));
    return address; 
  }
     
  return 0;
}


//==============================================================================
//PFLASH_Erase_Verify_Section
//==============================================================================
unsigned char PFLASH_Erase_Verify_Section(unsigned long int address, unsigned int number_of_phrases)
{
  //check if address is aligned (global address [2:0] = 000)
  if((address & 0x00000007) != 0)
    return MISALIGNED_ADDRESS; 
  
  while(FSTAT_CCIF == 0);   //wait if command in progress
  FSTAT = 0x30;             //clear ACCERR and PVIOL
  
  FCCOBIX = 0x00;
  FCCOB = 0x0300 | ((address & 0x00FF0000)>>16);
  
  FCCOBIX = 0x01;
  FCCOB = (address & 0x0000FFFF);
  
  FCCOBIX = 0x02;
  FCCOB = number_of_phrases;
  
  asm JSR Send_Command;
  
  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0)
    return ACCESS_ERROR;
  
  //check if phrases are erased and return result
  if(FSTAT_MGSTAT != 0)
    return NON_ERASED;
  else
    return ERASED;
}


//==============================================================================
//PFLASH_Program
//==============================================================================
unsigned char PFLASH_Program(unsigned long int address, unsigned int *ptr)
{
  unsigned char i;
  
  //check if address is aligned (global address [2:0] != 000)
  if((address & 0x00000007) != 0)
    return MISALIGNED_ADDRESS; 
  
  //check if the phrase is erased    
  if((PFLASH_Erase_Verify_Section(address, 1)) == NON_ERASED)
    return NON_ERASED;  
    
  while(FSTAT_CCIF == 0);   //wait if command in progress
  FSTAT = 0x30;             //clear ACCERR and PVIOL    
  
  FCCOBIX = 0x00;
  FCCOB = 0x0600 | ((address & 0x00030000)>>16);
  
  FCCOBIX = 0x01;
  FCCOB = (address & 0x0000FFFF);
  
  for(i=2; i<6; i++)  //fill data (4 words) to FCCOB register
  {
    FCCOBIX = i;
    FCCOB = *ptr;
    ptr++;  
  }
      
  asm JSR Send_Command;    
  
  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0)
    return ACCESS_ERROR;
  else    
    return OK;
}


//==============================================================================
//PFLASH_Erase_Sector
//==============================================================================
unsigned char PFLASH_Erase_Sector(unsigned long int address)
{
  //size of sector is 512B
  
  //check if address is aligned (global address [2:0] != 000)  
  if((address & 0x00000007) != 0)
    return MISALIGNED_ADDRESS; 
    
  while(FSTAT_CCIF == 0);   //wait if command in progress
  FSTAT = 0x30;             //clear ACCERR and PVIOL
  
  FCCOBIX = 0x00;
  FCCOB = 0x0A00 | ((address & 0x00030000)>>16);
  
  FCCOBIX = 0x01;
  FCCOB = (address & 0x0000FFF8);
   
  asm JSR Send_Command;
  
  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0)
    return ACCESS_ERROR;
  else    
    return OK;
}

//==============================================================================
//PFLASH_Read_Byte
//==============================================================================
unsigned char PFLASH_Read_Byte(unsigned long int address)
{
  unsigned char data8;
  data8 = *(unsigned char *far)address;
  return data8;
}

//==============================================================================
//PFLASH_Read_Word
//==============================================================================
unsigned int PFLASH_Read_Word(unsigned long int address)
{
  unsigned int data16;
  data16 = *(unsigned int *far)address;
  return data16;
}

//==============================================================================
//PFLASH_Init
//==============================================================================
void PFLASH_Init(unsigned char fdiv)
{
  //while(FSTAT_CCIF == 0);   //wait if command in progress  
  FCLKDIV = fdiv;           //osc = 16MHz    
}
